package uoc.javier.allpay;

import android.content.DialogInterface;
import android.content.Intent;
import android.support.annotation.NonNull;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.method.ScrollingMovementMethod;
import android.text.style.RelativeSizeSpan;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.thomashaertel.widget.MultiSpinner;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import uoc.javier.allpay.model.Item;
import uoc.javier.allpay.model.User;

public class EditItemActivity extends AppCompatActivity implements View.OnClickListener {
    private static final String TAG = "TFM:EditItemActivity";

    // UI references.
    private EditText etDescription;
    private EditText etPrice;
    private Button buttonEdit;
    private Button buttonReturn;
    private Button buttonRemove;
    private TextView tvLegend;

    private Spinner spnType;

    private MultiSpinner mspnReceivers;
    private ArrayAdapter<String> adapterReceivers;
    private TextView tvReceivers;

    private MultiSpinner mspnPayers;
    private ArrayAdapter<String> adapterPayers;
    private TextView tvPayers;

    // Receive the item from UserMovementsFragment
    private Bundle bundle;

    // Firebase
    private Map<String, Boolean> username_payers = new HashMap<>();
    private Map<String, Boolean> username_receivers = new HashMap<>();
    private DatabaseReference databaseDownload;
    private DatabaseReference databaseItemsHistorical;
    private String item_uid;

    // Item history
    private boolean item_history = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edit_item);

        etDescription = (EditText) findViewById(R.id.et_description);
        etPrice = (EditText) findViewById(R.id.et_price);

        buttonEdit = (Button) findViewById(R.id.btn_edit);
        buttonReturn = (Button) findViewById(R.id.btn_return);
        buttonRemove = (Button) findViewById(R.id.btn_remove);

        tvLegend = (TextView) findViewById(R.id.tv_legend);

        // Spinner type
        spnType = (Spinner) findViewById(R.id.spn_type);
        // Create an ArrayAdapter using the string array and the developed spinner layout
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.product_types, R.layout.my_spinner);
        // Specify the layout to use when the list of choices appears
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        // Apply the adapter to the spinner
        spnType.setAdapter(adapter);

        // MultiSpinner payers
        mspnPayers = (MultiSpinner) findViewById(R.id.mspn_payers);
        adapterPayers = new ArrayAdapter<String>(this, R.layout.my_spinner);
        mspnPayers.setVisibility(View.GONE);
        tvPayers = (TextView) findViewById(R.id.tv_payers);
        tvPayers.setMovementMethod(new ScrollingMovementMethod());

        // MultiSpinner receivers
        mspnReceivers = (MultiSpinner) findViewById(R.id.mspn_receivers);
        adapterReceivers = new ArrayAdapter<String>(this, R.layout.my_spinner);
        mspnReceivers.setVisibility(View.GONE);
        tvReceivers = (TextView) findViewById(R.id.tv_receivers);
        tvReceivers.setMovementMethod(new ScrollingMovementMethod());

        // Download users from Firebase
        DatabaseReference databaseUsers;
        databaseUsers = FirebaseDatabase.getInstance().getReference("users");

        databaseUsers.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                 for(DataSnapshot user : dataSnapshot.getChildren()){
//                     Log.d(TAG, "users "+ user.getKey());
                     adapterPayers.add(user.getKey());
                     adapterReceivers.add(user.getKey());
                 }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });

        adapterPayers.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mspnPayers.setAdapter(adapterPayers, false, onSelectedListenerPayers);

        adapterReceivers.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mspnReceivers.setAdapter(adapterReceivers, false, onSelectedListenerReceivers);

        // Set listeners
        buttonEdit.setOnClickListener(this);
        buttonReturn.setOnClickListener(this);
        buttonRemove.setOnClickListener(this);
        tvReceivers.setOnClickListener(this);
        tvPayers.setOnClickListener(this);

        // Receive information
        bundle = getIntent().getExtras();
        item_uid = bundle.getString("item_uid");

        // Check if it is called by ItemsHistoricalActivity
        item_history = bundle.getBoolean("item_history");

        String database_path = "items";

        // If it is the item historic view it makes some changes
        if(item_history){
            tvLegend.setText(getResources().getString(R.string.item_history));

            buttonEdit.setVisibility(View.GONE);
            buttonRemove.setVisibility(View.GONE);

            buttonEdit.setEnabled(false);
            buttonRemove.setEnabled(false);
            etDescription.setEnabled(false);
            etPrice.setEnabled(false);
            spnType.setEnabled(false);
            mspnPayers.setAdapter(null, false, onSelectedListenerPayers);
            mspnReceivers.setAdapter(null, false, onSelectedListenerPayers);

            database_path = "items_historical";
        }
        // items_historical database needed to update the item in the history
        else{
            databaseItemsHistorical = FirebaseDatabase.getInstance().getReference("items_historical");
        }

        databaseDownload = FirebaseDatabase.getInstance().getReference(database_path);

        // Download items
        databaseDownload.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                DataSnapshot dataSnapshot_item = dataSnapshot.child(item_uid);

                // Exception after removing item
                try {
                    etDescription.setText(dataSnapshot_item.child("description").getValue(String.class));
                    etPrice.setText(dataSnapshot_item.child("price").getValue(String.class));

                    String type = dataSnapshot_item.child("type").getValue(String.class);

                    if (type.equals(getResources().getString(R.string.product)))
                        spnType.setSelection(0);
                    else
                        spnType.setSelection(1);

                    // MultiSpinner
                    List<String> payers_users = new ArrayList<>();
                    List<String> payers_receivers = new ArrayList<>();

                    // Load payers from item
                    for (DataSnapshot payer : dataSnapshot_item.child("payers_users").getChildren()) {
                        payers_users.add(payer.getKey().toString());
                        username_payers.put(payer.getKey().toString(), true);
                    }

                    // Load in Multispinner
                    boolean[] selectedItemsPayers = new boolean[adapterPayers.getCount()];
                    for (int i = 0; i < payers_users.size(); i++) {
                        for (int j = 0; j < adapterPayers.getCount(); j++) {
                            if (payers_users.get(i).equals(adapterPayers.getItem(j)))
                                selectedItemsPayers[j] = true; // select second item
                        }
                    }
                    mspnPayers.setSelected(selectedItemsPayers);
                    // Delete "[" and "]" characters
                    tvPayers.setText(payers_users.toString().replace("[", "").replace("]", ""));

                    // Load receivers from item
                    for (DataSnapshot receiver : dataSnapshot_item.child("receivers_users").getChildren()) {
                        payers_receivers.add(receiver.getKey().toString());
                        username_receivers.put(receiver.getKey().toString(), true);

                    }

                    // Load in Multispinner
                    boolean[] selectedItemsReceivers = new boolean[adapterReceivers.getCount()];
                    for (int i = 0; i < payers_receivers.size(); i++) {
                        for (int j = 0; j < adapterReceivers.getCount(); j++) {
                            if (payers_receivers.get(i).equals(adapterReceivers.getItem(j)))
                                selectedItemsReceivers[j] = true; // select second item
                        }
                    }
                    mspnReceivers.setSelected(selectedItemsReceivers);
                    // Delete "[" and "]" characters
                    tvReceivers.setText(payers_receivers.toString().replace("[", "").replace("]", ""));
                }catch (Exception e){

                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });
    }

    private MultiSpinner.MultiSpinnerListener onSelectedListenerReceivers = new MultiSpinner.MultiSpinnerListener() {
        public void onItemsSelected(boolean[] selected) {
            StringBuilder builder = new StringBuilder();

            // New username_receivers
            username_receivers = new HashMap<>();
            for (int i = 0; i < selected.length; i++) {
                if (selected[i]) {
                    builder.append(adapterReceivers.getItem(i)).append(" ; ");
                    username_receivers.put(adapterPayers.getItem(i), true);
                }
            }
            tvReceivers.setText(builder.toString());
            //Toast.makeText(getActivity(), builder.toString(), Toast.LENGTH_SHORT).show();
        }
    };

    private MultiSpinner.MultiSpinnerListener onSelectedListenerPayers = new MultiSpinner.MultiSpinnerListener() {
        public void onItemsSelected(boolean[] selected) {
            StringBuilder builder = new StringBuilder();

            // New username_payers
            username_payers = new HashMap<>();
            for (int i = 0; i < selected.length; i++) {
                if (selected[i]) {
                    builder.append(adapterPayers.getItem(i)).append(" ; ");
                    username_payers.put(adapterPayers.getItem(i), true);
                }
            }
            tvPayers.setText(builder.toString());
            //Toast.makeText(getActivity(), builder.toString(), Toast.LENGTH_SHORT).show();
        }
    };

    public void onClick(View v) {
        if (v == buttonEdit){
            // Clean error message and icon from TextViews
            tvPayers.setError(null);
            tvReceivers.setError(null);

            // Check that each requiered field is filled
            if(!checkRequeriedFields(new EditText[]{etDescription, etPrice})){
                showToast(getResources().getString(R.string.complete_fields));
            }

            // Check that at least one user is the payer and one user is the receiver
            else if(username_payers.isEmpty()){
                tvPayers.setError(getResources().getString(R.string.complete_fields));
                showToast(getResources().getString(R.string.complete_fields));
            }

            else if (username_receivers.isEmpty()){
                tvReceivers.setError(getResources().getString(R.string.complete_fields));
                showToast(getResources().getString(R.string.complete_fields));
            }

            else{
                // Edit item
                Item update_item = new Item(etDescription.getText().toString(),
                        spnType.getSelectedItem().toString(),
                        etPrice.getText().toString(),
                        null, null);


                // Update item
                databaseDownload.child(item_uid).setValue(update_item)
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                //Log.d(TAG, "starting item updated");
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                showToast(getResources().getString(R.string.add_item_wrong));
                            }
                        });

                // Add the username_payers and username_receivers components to the updated item
                databaseDownload.child(item_uid).child("payers_users").setValue(username_payers)
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                //Log.d(TAG, "payers_users added");
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                showToast(getResources().getString(R.string.update_item_wrong));
                            }
                        });

                databaseDownload.child(item_uid).child("receivers_users").setValue(username_receivers)
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                //Log.d(TAG, "receivers_users added");
                                showToast(getResources().getString(R.string.update_item_right));
                                // Go to the main fragment of the TabbetActivity and clean all the fields
                                goTabbedActivity();
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                showToast(getResources().getString(R.string.update_item_wrong));
                            }
                        });

                // Update the same item in the item_historical database
                databaseItemsHistorical.child(item_uid).setValue(update_item)
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                //Log.d(TAG, "starting item updated");
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                showToast(getResources().getString(R.string.add_item_wrong));
                            }
                        });

                databaseItemsHistorical.child(item_uid).child("payers_users").setValue(username_payers)
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                //Log.d(TAG, "payers_users added");
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                showToast(getResources().getString(R.string.update_item_wrong));
                            }
                        });

                databaseItemsHistorical.child(item_uid).child("receivers_users").setValue(username_receivers)
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                //Log.d(TAG, "receivers_users added");
                                showToast(getResources().getString(R.string.update_item_right));
                                // Go to the main fragment of the TabbetActivity and clean all the fields
                                goTabbedActivity();
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                showToast(getResources().getString(R.string.update_item_wrong));
                            }
                        });
            }
        }
        else if(v == buttonReturn){
            if(item_history){
                // Go to the ItemsHistoricalActivity and clean all the fields
                Intent intent = new Intent(this, ItemsHistoricalActivity.class);
                startActivity(intent);
            }
            else {
                // Go to the main fragment of the TabbetActivity and clean all the fields
                goTabbedActivity();
            }
        }
        else if(v == buttonRemove) {
            // Ask to be sure and then remove the item
            askRemoveConfirmation();
        }

        else if(v == tvReceivers){
            mspnReceivers.performClick();
        }

        else if(v == tvPayers){
            mspnPayers.performClick();
        }
    }

    public void askRemoveConfirmation(){
        DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which){
                    case DialogInterface.BUTTON_POSITIVE:
                        removeItem();
                        break;

                    case DialogInterface.BUTTON_NEGATIVE:
                        break;
                }
            }
        };

        AlertDialog builder = new AlertDialog.Builder(this)
                .setMessage(getResources().getString(R.string.remove_item_confirmation))
                .setPositiveButton(getResources().getString(R.string.option_yes), dialogClickListener)
                .setNegativeButton(getResources().getString(R.string.option_no), dialogClickListener).show();

        TextView textView = (TextView) builder.findViewById(android.R.id.message);
        textView.setTextSize(20);
    }

    public void removeItem() {
        databaseDownload.child(item_uid).removeValue()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        showToast(getResources().getString(R.string.remove_item_right));
                        goTabbedActivity();
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        showToast(getResources().getString(R.string.remove_item_wrong));
                    }
                });

        // Remove the same item from the items_historical database
        databaseItemsHistorical.child(item_uid).removeValue()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        showToast(getResources().getString(R.string.remove_item_right));
                        goTabbedActivity();
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        showToast(getResources().getString(R.string.remove_item_wrong));
                    }
                });
    }

    public void goTabbedActivity(){
        // Go to the main fragment of the TabbetActivity and clean all the fields
        Intent intent = new Intent(this, TabbedActivity.class);
        startActivity(intent);
    }

    public boolean checkRequeriedFields(EditText[] fields){
        EditText currentField;
        for(int i=0; i<fields.length; i++){
            currentField = fields[i];
            if(currentField.getText().toString().length() <=0){
                currentField.setError(getResources().getString(R.string.complete_fields));
                return false;
            }
        }
        return true;
    }

    public void showToast(String str){
        // Configuration size of text
        SpannableString ss =  new SpannableString(str);
        ss.setSpan(new RelativeSizeSpan(1.5f), 0, ss.length(), 0);

        // Show the message
        Toast.makeText(this, ss, Toast.LENGTH_LONG).show();
    }
}
