package uoc.javier.allpay;

import android.content.DialogInterface;
import android.content.Intent;
import android.support.annotation.NonNull;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.style.RelativeSizeSpan;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import uoc.javier.allpay.model.Goal;

public class GoalActivity extends AppCompatActivity implements View.OnClickListener {
    private static final String TAG = "TFM:GoalActivity";

    // UI references.
    private EditText etDescription;
    private EditText etRaised;
    private EditText etGoal;
    private Button buttonEdit;
    private Button buttonReturn;
    private Button buttonRemove;

    // Receive the item from UserMovementsFragment
    private Bundle bundle;
    private String uid;

    // Firebase
    DatabaseReference mDatabase;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_goal);

        etDescription = (EditText) findViewById(R.id.et_description);
        etRaised = (EditText) findViewById(R.id.et_raised);
        etGoal = (EditText) findViewById(R.id.et_goal);

        buttonEdit = (Button) findViewById(R.id.btn_edit);
        buttonReturn = (Button) findViewById(R.id.btn_return);
        buttonRemove = (Button) findViewById(R.id.btn_remove);

        mDatabase = FirebaseDatabase.getInstance().getReference("goals");

        // Set listeners
        buttonEdit.setOnClickListener(this);
        buttonReturn.setOnClickListener(this);
        buttonRemove.setOnClickListener(this);

        bundle = getIntent().getExtras();

        // Edit goal
        if(bundle != null) {
            uid = bundle.getString("goal_uid");
            downloadThisGoal();
        }

        // Add new goal
        else{
            buttonEdit.setText(getResources().getString(R.string.add));
            buttonRemove.setEnabled(false);
            buttonRemove.setVisibility(View.GONE);

            TextView legend = (TextView)findViewById(R.id.tv_legend);
            legend.setText(getResources().getString(R.string.legend_add_goal));
        }

    }

    public void downloadThisGoal(){
        // Download goal from Firebase
        mDatabase.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                Goal goal = dataSnapshot.child(uid).getValue(Goal.class);

                // Exception after removing goal
                try {
                    etDescription.setText(goal.getSubject());
                    etRaised.setText(goal.getRaised() + " €");
                    etGoal.setText(goal.getGoal() + " €");
                }catch (Exception e){

                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });
    }

    public void onClick(View v) {
        if (v == buttonEdit){
            // Check that each requiered field is filled
            if(!checkRequeriedFields(new EditText[]{etDescription, etRaised, etGoal})){
                showToast(getResources().getString(R.string.complete_fields));
            }

            // Add new goal
            else if(buttonEdit.getText().equals(getResources().getString(R.string.add))){
                Goal add_goal = new Goal(etDescription.getText().toString(),
                        etRaised.getText().toString().replace("€", ""),
                        etGoal.getText().toString().replace("€", ""));

                mDatabase.push().setValue(add_goal)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                showToast(getResources().getString(R.string.add_goal_right));
                                goTabbedActivity();
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                showToast(getResources().getString(R.string.add_goal_wrong));
                            }
                        });
            }

            // Edit goal
            else if(buttonEdit.getText().equals(getResources().getString(R.string.edit))){
                // Remove € symbol
                Goal add_goal = new Goal(etDescription.getText().toString(),
                                        etRaised.getText().toString().replace("€", ""),
                                        etGoal.getText().toString().replace("€", ""));
                mDatabase.child(uid).setValue(add_goal)
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                showToast(getResources().getString(R.string.update_goal_right));
                                goTabbedActivity();
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                showToast(getResources().getString(R.string.update_goal_wrong));
                            }
                        });
            }
        }
        else if(v == buttonReturn){
            goTabbedActivity();
        }

        else if(v == buttonRemove) {
            // Ask to be sure and then remove the goal
            askRemoveConfirmation();
        }
    }

    public void askRemoveConfirmation(){
        DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which){
                    case DialogInterface.BUTTON_POSITIVE:
                        removeGoal();
                        break;

                    case DialogInterface.BUTTON_NEGATIVE:
                        break;
                }
            }
        };

        AlertDialog builder = new AlertDialog.Builder(this)
                .setMessage(getResources().getString(R.string.remove_goal_confirmation))
                .setPositiveButton(getResources().getString(R.string.option_yes), dialogClickListener)
                .setNegativeButton(getResources().getString(R.string.option_no), dialogClickListener).show();

        TextView textView = (TextView) builder.findViewById(android.R.id.message);
        textView.setTextSize(20);
    }

    public void removeGoal() {
        mDatabase.child(uid).removeValue()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        showToast(getResources().getString(R.string.remove_goal_right));
                        goTabbedActivity();
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        showToast(getResources().getString(R.string.remove_goal_wrong));
                    }
                });
    }

    public boolean checkRequeriedFields(EditText[] fields){
        EditText currentField;
        for(int i=0; i<fields.length; i++){
            currentField = fields[i];
            if(currentField.getText().toString().length() <=0){
                currentField.setError(getResources().getString(R.string.complete_fields));
                return false;
            }
        }
        return true;
    }

    public void goTabbedActivity(){
        // Go to the main fragment of the TabbetActivity and clean all the fields
        Intent intent = new Intent(this, TabbedActivity.class);
        startActivity(intent);
    }

    public void showToast(String str){
        // Configuration size of text
        SpannableString ss =  new SpannableString(str);
        ss.setSpan(new RelativeSizeSpan(1.5f), 0, ss.length(), 0);

        // Show the message
        Toast.makeText(this, ss, Toast.LENGTH_LONG).show();
    }
}
