package uoc.javier.allpay;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.util.DiffUtil;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import uoc.javier.allpay.model.Goal;
import uoc.javier.allpay.model.Item;
import uoc.javier.allpay.utils.GoalDiffCallback;

/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link GoalsFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 * Use the {@link GoalsFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class GoalsFragment extends Fragment {
    private static final String TAG = "TFM:GoalsFragment";

    // TODO: Rename parameter arguments, choose names that match
    // the fragment initialization parameters, e.g. ARG_ITEM_NUMBER
    private static final String ARG_PARAM1 = "param1";
    private static final String ARG_PARAM2 = "param2";

    // TODO: Rename and change types of parameters
    private String mParam1;
    private String mParam2;

    private OnFragmentInteractionListener mListener;

    private RecyclerView recyclerView;
    private GoalsFragment.SimpleItemRecyclerViewAdapter adapter;
    private SwipeRefreshLayout swipeContainer;

    private FirebaseDatabase database;
    private DatabaseReference myRef;

    public GoalsFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @param param1 Parameter 1.
     * @param param2 Parameter 2.
     * @return A new instance of fragment GoalsFragment.
     */
    // TODO: Rename and change types and number of parameters
    public static GoalsFragment newInstance(String param1, String param2) {
        GoalsFragment fragment = new GoalsFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PARAM1, param1);
        args.putString(ARG_PARAM2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            mParam1 = getArguments().getString(ARG_PARAM1);
            mParam2 = getArguments().getString(ARG_PARAM2);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_goals, container, false);
    }

    @Override
    public void onViewCreated(final View view, Bundle savedInstanceState) {
        recyclerView = (RecyclerView)view.findViewById(R.id.goals_list);
        StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(1, StaggeredGridLayoutManager.VERTICAL);
        recyclerView.setLayoutManager(staggeredGridLayoutManager);
        setupRecyclerView(recyclerView);

        database = FirebaseDatabase.getInstance();

        swipeContainer = (SwipeRefreshLayout) view.findViewById(R.id.swipeContainer);
        // Setup refresh listener which triggers new data loading
        swipeContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refreshGoalsList();
            }
        });

        // Configure the refreshing colors
        swipeContainer.setColorSchemeResources(android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);

        downloadGoals();

        // Add new goal
        FloatingActionButton fab = (FloatingActionButton) view.findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Context context = view.getContext();
                Intent intent = new Intent(context, GoalActivity.class);
                context.startActivity(intent);
            }
        });
    }

    private void downloadGoals() {
        swipeContainer.setRefreshing(true);

        myRef = database.getReference("goals");
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getGoalsFromDataSnapshot(dataSnapshot);
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });

        DatabaseReference connectedRef = database.getReference(".info/connected");
        connectedRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot snapshot) {
                boolean connected = snapshot.getValue(Boolean.class);
                if (!connected) {
                    Log.d(TAG, "Failed to connect");
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG, "Listener was cancelled");
            }
        });
    }

    private void refreshGoalsList() {
        if (myRef != null) {
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    getGoalsFromDataSnapshot(dataSnapshot);
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    // Failed to read value
                    Log.d(TAG, "Failed to read value.", databaseError.toException());
                }
            });
        } else {
            Log.d(TAG, "My Ref null");
        }
    }

    private void getGoalsFromDataSnapshot(DataSnapshot dataSnapshot) {
        // This method is called once with the initial value and again
        // whenever data at this location is updated.

        ArrayList<Goal> values = new ArrayList<>();
        Goal goal;

        for (DataSnapshot goalValues : dataSnapshot.getChildren()){
            //values.add(dataValues.getValue(Goal.class));
            goal = goalValues.getValue(Goal.class);
            goal.setUid(goalValues.getKey());

            values.add(goal);
        }

        // To show the last ones first
        Collections.reverse(values);

        // Update values
        adapter.setItems(values);
        swipeContainer.setRefreshing(false);

        // Check if the fragment is currently added to its activity.
        if(isAdded())
            Snackbar.make(swipeContainer, getResources().getString(R.string.update_goals), Snackbar.LENGTH_LONG).show();
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
//        adapter = new SimpleItemRecyclerViewAdapter(Goal.GOALS_EXAMPLE, getContext());
        adapter = new SimpleItemRecyclerViewAdapter(new ArrayList<Goal>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<GoalsFragment.SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<Goal> mValues;

        public SimpleItemRecyclerViewAdapter(List<Goal> goals) {
            mValues = goals;
        }

        public void setItems(List<Goal> goals) {
            DiffUtil.DiffResult diffResult = DiffUtil.calculateDiff(new GoalDiffCallback(mValues, goals));
            diffResult.dispatchUpdatesTo(this);
            mValues = goals;
        }

        @Override
        public GoalsFragment.SimpleItemRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext()).inflate(R.layout.goals_list_content, parent, false);
            return new GoalsFragment.SimpleItemRecyclerViewAdapter.ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final GoalsFragment.SimpleItemRecyclerViewAdapter.ViewHolder holder, final int position) {
            holder.mGoal = mValues.get(position);
            holder.mSubject.setText(holder.mGoal.getSubject());
            holder.mRaised.setText(holder.mGoal.getRaised() + " €");
            holder.mGoal_text.setText(holder.mGoal.getGoal() + " €");

            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    //Log.d(TAG, "uid: "+holder.mGoal.getUid());
                    Context context = v.getContext();
                    Intent intent = new Intent(context, GoalActivity.class);
                    intent.putExtra("goal_uid", holder.mGoal.getUid());
                    context.startActivity(intent);
                }
            });
        }

        @Override
        public int getItemCount() {
            return mValues.size();
        }

        class ViewHolder extends RecyclerView.ViewHolder {
            final View mView;
            final TextView mSubject;
            final TextView mRaised;
            final TextView mGoal_text;
            Goal mGoal;

            ViewHolder(View view) {
                super(view);
                mView = view;
                mSubject = (TextView) view.findViewById(R.id.subject_content);
                mRaised = (TextView) view.findViewById(R.id.raised_content);
                mGoal_text = (TextView) view.findViewById(R.id.goal_content);
            }
        }
    }

    // TODO: Rename method, update argument and hook method into UI event
    public void onButtonPressed(Uri uri) {
        if (mListener != null) {
            mListener.onFragmentInteraction(uri);
        }
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            //throw new RuntimeException(context.toString() + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }
}
