package uoc.javier.allpay;

import android.content.Context;
import android.content.Intent;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.util.DiffUtil;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.text.SpannableString;
import android.text.style.RelativeSizeSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import uoc.javier.allpay.model.Item;
import uoc.javier.allpay.utils.ItemDiffCallback;

public class ItemsHistoricalActivity extends AppCompatActivity implements View.OnClickListener {
    private static final String TAG = "TFM:ItemsHistorical";

    // UI references.
    private Button buttonReturn;

    private RecyclerView recyclerView;
    private ItemsHistoricalActivity.SimpleItemRecyclerViewAdapter adapter;

    private SwipeRefreshLayout swipeContainer;

    // Firebase
    private FirebaseDatabase database;
    private DatabaseReference myRef;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_items_historical);

        buttonReturn = (Button) findViewById(R.id.btn_return);

        recyclerView = (RecyclerView) findViewById(R.id.itemshistorical_list);
        StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(1, StaggeredGridLayoutManager.VERTICAL);
        recyclerView.setLayoutManager(staggeredGridLayoutManager);
        setupRecyclerView(recyclerView);

        database = FirebaseDatabase.getInstance();

        swipeContainer = (SwipeRefreshLayout) findViewById(R.id.swipeContainer);
        // Setup refresh listener which triggers new data loading
        swipeContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refreshItemsList();
            }
        });

        // Configure the refreshing colors
        swipeContainer.setColorSchemeResources(android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);

        downloadItems();

        // Set listeners
        buttonReturn.setOnClickListener(this);
    }

    public void onClick(View v) {
        if(v == buttonReturn){
            // Go to the main fragment of the TabbetActivity and clean all the fields
            Intent intent = new Intent(this, TabbedActivity.class);
            startActivity(intent);
        }
    }

    private void downloadItems() {
        swipeContainer.setRefreshing(true);

        myRef = database.getReference("items_historical");
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getItemsFromDataSnapshot(dataSnapshot);
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });

        DatabaseReference connectedRef = database.getReference(".info/connected");
        connectedRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot snapshot) {
                boolean connected = snapshot.getValue(Boolean.class);
                if (!connected) {
                    Log.d(TAG, "Failed to connect");
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG, "Listener was cancelled");
            }
        });
    }

    private void refreshItemsList() {
        if (myRef != null) {
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    getItemsFromDataSnapshot(dataSnapshot);
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    // Failed to read value
                    Log.d(TAG, "Failed to read value.", databaseError.toException());
                }
            });
        } else {
            Log.d(TAG, "My Ref null");
        }
    }

    private void getItemsFromDataSnapshot(DataSnapshot dataSnapshot) {
        // This method is called once with the initial value and again
        // whenever data at this location is updated.
        ArrayList<Item> values = new ArrayList<>();

        Item item;
        List<String> payers_users;
        List<String> receivers_users;

        // Get each item
        for (DataSnapshot dataValues : dataSnapshot.getChildren()){
            item = new Item();
            payers_users = new ArrayList<>();
            receivers_users = new ArrayList<>();

            item.setUid(dataValues.getKey());

            // Log.d(TAG, dataValues.toString());
            // Log.d(TAG, "key " + dataValues.child("price").getValue(String.class));
            item.setDescription(dataValues.child("description").getValue(String.class));
            item.setPrice(dataValues.child("price").getValue(String.class));
            item.setType(dataValues.child("type").getValue(String.class));

            for(DataSnapshot payer : dataValues.child("payers_users").getChildren()){
//                Log.d(TAG, "hijo " + payers.getKey().toString());
                payers_users.add(payer.getKey());
            }

            for(DataSnapshot receiver : dataValues.child("receivers_users").getChildren()){
//                Log.d(TAG, "hijo " + receivers.getKey().toString());
                receivers_users.add(receiver.getKey());
            }

            item.setPayers_users(payers_users);
            item.setReceivers_users(receivers_users);

            values.add(item);
        }

        // To show the last ones first
        Collections.reverse(values);

        // Update values
        adapter.setItems(values);
        swipeContainer.setRefreshing(false);

        Snackbar.make(swipeContainer, getResources().getString(R.string.update_items_history), Snackbar.LENGTH_LONG).show();
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
//        adapter = new SimpleItemRecyclerViewAdapter(Item.ITEMS_EXAMPLE, getContext());
        adapter = new ItemsHistoricalActivity.SimpleItemRecyclerViewAdapter(new ArrayList<Item>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<ItemsHistoricalActivity.SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<Item> mValues;


        public SimpleItemRecyclerViewAdapter(List<Item> items) {
            mValues = items;
        }

        public void setItems(List<Item> items) {
            DiffUtil.DiffResult diffResult = DiffUtil.calculateDiff(new ItemDiffCallback(mValues, items));
            diffResult.dispatchUpdatesTo(this);
            mValues = items;
        }

        @Override
        public ItemsHistoricalActivity.SimpleItemRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext()).inflate(R.layout.itemshistorical_list_content, parent, false);
            return new ItemsHistoricalActivity.SimpleItemRecyclerViewAdapter.ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ItemsHistoricalActivity.SimpleItemRecyclerViewAdapter.ViewHolder holder, final int position) {
            holder.mItem = mValues.get(position);
            holder.mSubject.setText(holder.mItem.getDescription());
            holder.mPrice.setText(holder.mItem.getPrice() + " €");

            // Delete "[" and "]" characters
            holder.mFrom.setText(holder.mItem.getPayers_users().toString().replace("[", "").replace("]", ""));
            holder.mTo.setText(holder.mItem.getReceivers_users().toString().replace("[", "").replace("]", ""));

            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Context context = v.getContext();
                    Intent intent = new Intent(context, EditItemActivity.class);
                    intent.putExtra("item_uid", holder.mItem.getUid());
                    // Call activity with item history view
                    intent.putExtra("item_history", true);
                    context.startActivity(intent);
                }
            });
        }

        @Override
        public int getItemCount() {
            return mValues.size();
        }

        class ViewHolder extends RecyclerView.ViewHolder {
            final View mView;
            final TextView mFrom;
            final TextView mSubject;
            final TextView mPrice;
            final TextView mTo;
            Item mItem;

            ViewHolder(View view) {
                super(view);
                mView = view;
                mFrom = (TextView) view.findViewById(R.id.from_content);
                mSubject = (TextView) view.findViewById(R.id.subject_content);
                mPrice = (TextView) view.findViewById(R.id.price_content);
                mTo = (TextView) view.findViewById(R.id.to_content);
            }
        }
    }

    public void showToast(String str){
        // Configuration size of text
        SpannableString ss =  new SpannableString(str);
        ss.setSpan(new RelativeSizeSpan(1.5f), 0, ss.length(), 0);

        // Show the message
        Toast.makeText(this, ss, Toast.LENGTH_LONG).show();
    }
}
