package uoc.javier.allpay;

import android.Manifest;
import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.TargetApi;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.support.annotation.NonNull;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AppCompatActivity;
import android.app.LoaderManager.LoaderCallbacks;

import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;

import android.os.Build;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.text.SpannableString;
import android.text.TextUtils;
import android.text.style.RelativeSizeSpan;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.List;

import uoc.javier.allpay.model.Item;
import uoc.javier.allpay.model.User;

import static android.Manifest.permission.READ_CONTACTS;

/**
 * A login screen that offers login via email/password.
 */
public class LoginActivity extends AppCompatActivity implements View.OnClickListener{
    private static final String TAG = "TFM:LoginActivity";

    /**
     * Id to identity READ_CONTACTS permission request.
     */
    // Request code permissions
    private final int REQUEST_PERMISSION_ALL = 100;
    private final String[] PERMISSIONS = {
            Manifest.permission.INTERNET
    };
    private List<String> remainingPermissions = new ArrayList<>();

    private FirebaseAuth mAuth;

    /**
     * Keep track of the login task to ensure we can cancel it if requested.
     */
    private UserLoginTask mAuthTask = null;

    // UI references.
    private EditText etEmail;
    private EditText etPassword;
    private TextView tvLegend;
    private View vProgressView;
    private View vLoginFormView;
    private Button buttonEnter;
    private Button buttonNew_user;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);

        // Set views
        etEmail = (EditText) findViewById(R.id.et_email);
        etPassword = (EditText) findViewById(R.id.et_password);
        tvLegend = (TextView) findViewById(R.id.tv_legend);

        // TODO TESTING - En el futuro, cuando se puedan dar de alta más usuarios reales en el apartado "Authentication" de Firebase, esto no será necesario
        etEmail.setText("javierparra.allpay@gmail.com");
        etPassword.setText("tfmallpay");


        buttonEnter = (Button) findViewById(R.id.btn_enter);
        buttonNew_user = (Button) findViewById(R.id.btn_new_user);
        vLoginFormView = findViewById(R.id.rl_login_form);
        vProgressView = findViewById(R.id.login_progress);

        // Set listeners
        buttonEnter.setOnClickListener(this);
        buttonNew_user.setOnClickListener(this);
    }

    @Override
    public void onClick(View v){
        if(v == buttonEnter){
            // Check that each requiered field is filled
            if(!checkRequeriedFields(new EditText[]{etEmail, etPassword})){
                showToast(getResources().getString(R.string.complete_fields));
            }
            else{
                // Check permissions
                if (!hasPermissions(PERMISSIONS)) {
                    ActivityCompat.requestPermissions(this, remainingPermissions.toArray(new String[remainingPermissions.size()]), REQUEST_PERMISSION_ALL);
                }
                else {
                    // Check email - password
                    attemptLogin();
                }
            }
        }else if(v == buttonNew_user){
            // Go to NewUserActivity
            Intent intent = new Intent(this, NewUserActivity.class);
            startActivity(intent);
            finish();
        }
    }

    public boolean checkRequeriedFields(EditText[] fields){
        EditText currentField;
        for(int i=0; i<fields.length; i++){
            currentField = fields[i];
            if(currentField.getText().toString().length() <=0){
                currentField.setError(getResources().getString(R.string.complete_fields));
                return false;
            }
        }
        return true;
    }

    // START PERMISSIONS
    private boolean hasPermissions(String... permissions) {
        boolean permisionsOk = true;
        if (permissions != null) {
            remainingPermissions.clear();
            for (String permission : permissions) {
                if (ActivityCompat.checkSelfPermission(this, permission) != PackageManager.PERMISSION_GRANTED) {
                    remainingPermissions.add(permission);
                    permisionsOk = false;
                }
            }
        }
        return permisionsOk;
    }

    @Override
    public void onRequestPermissionsResult ( int requestCode, String permissions[], int[] grantResults){
        switch (requestCode) {
            case REQUEST_PERMISSION_ALL: {
                if (grantResults.length > 0) {
                    for (int grantResult : grantResults) {
                        if (grantResult != PackageManager.PERMISSION_GRANTED) {
                            showToast(getResources().getString(R.string.permissions_app));
                            return;
                        }
                    }
                    attemptLogin();
                } else {
                    showToast(getResources().getString(R.string.permissions_app));
                }
            }
        }
    }

    // END PERMISSIONS

    public void showToast(String str){
        // Configuration size of text
        SpannableString ss =  new SpannableString(str);
        ss.setSpan(new RelativeSizeSpan(1.5f), 0, ss.length(), 0);

        // Show the message
        Toast.makeText(this, ss, Toast.LENGTH_LONG).show();
    }

    /**
     * Attempts to sign in or register the account specified by the login form.
     * If there are form errors (invalid email, missing fields, etc.), the
     * errors are presented and no actual login attempt is made.
     */
    private void attemptLogin() {
        if (mAuthTask != null) {
            return;
        }

        // Reset errors.
        etEmail.setError(null);
        etPassword.setError(null);

        // Store values at the time of the login attempt.
        String email = etEmail.getText().toString();
        String password = etPassword.getText().toString();

        boolean cancel = false;
        View focusView = null;

        // Check for a valid password, if the user entered one.
        if (!TextUtils.isEmpty(password) && !isPasswordValid(password)) {
            etPassword.setError(getString(R.string.error_invalid_password));
            focusView = etPassword;
            cancel = true;
        }

        // Check for a valid email address.
        if (TextUtils.isEmpty(email)) {
            etEmail.setError(getString(R.string.error_field_required));
            focusView = etEmail;
            cancel = true;
        } else if (!isEmailValid(email)) {
            etEmail.setError(getString(R.string.error_invalid_email));
            focusView = etEmail;
            cancel = true;
        }

        if (cancel) {
            // There was an error; don't attempt login and focus the first
            // form field with an error.
            focusView.requestFocus();
        } else {
            // Show a progress spinner, and kick off a background task to
            // perform the user login attempt.
            showProgress(true);
            mAuthTask = new UserLoginTask(email, password);
            mAuthTask.execute((Void) null);
        }
    }

    private boolean isEmailValid(String email) {
        return email.contains("@");
    }

    private boolean isPasswordValid(String password) {
        return password.length() > 4;
    }

    /**
     * Shows the progress UI and hides the login form.
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
    private void showProgress(final boolean show) {
        // On Honeycomb MR2 we have the ViewPropertyAnimator APIs, which allow
        // for very easy animations. If available, use these APIs to fade-in
        // the progress spinner.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
            int shortAnimTime = getResources().getInteger(android.R.integer.config_shortAnimTime);

            etEmail.setVisibility(show ? View.GONE : View.VISIBLE);
            tvLegend.setVisibility(show ? View.GONE : View.VISIBLE);
            vLoginFormView.animate().setDuration(shortAnimTime).alpha(
                    show ? 0 : 1).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    vLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
                }
            });

            vProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            vProgressView.animate().setDuration(shortAnimTime).alpha(
                    show ? 1 : 0).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    vProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
                }
            });
        } else {
            // The ViewPropertyAnimator APIs are not available, so simply show
            // and hide the relevant UI components.
            vProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            vLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
        }
    }

    /**
     * Represents an asynchronous login/registration task used to authenticate
     * the user.
     */
    public class UserLoginTask extends AsyncTask<Void, Void, Void> {
        private final String mEmail;
        private final String mPassword;

        UserLoginTask(String email, String password) {
            mEmail = email;
            mPassword = password;
        }

        @Override
        protected Void doInBackground(Void... params) {
            mAuth = FirebaseAuth.getInstance();

            mAuth.signInWithEmailAndPassword(mEmail, mPassword)
                    .addOnCompleteListener(new OnCompleteListener<AuthResult>() {
                        @Override
                        public void onComplete(@NonNull Task<AuthResult> task) {
                            Log.d(TAG, "signIn:onComplete:" + task.isSuccessful());
                            mAuthTask = null;
                            showProgress(false);
                            if (task.isSuccessful()) {
                                // Download user from Firebase and go to TabbedActivity
                                goToTabbedActivity(mEmail);
                            } else {
                                etPassword.setError(getString(R.string.error_incorrect_password));
                                etPassword.requestFocus();
                            }
                        }
                    });
            return null;
        }

        @Override
        protected void onCancelled() {
            mAuthTask = null;
            showProgress(false);
        }
    }

    public void goToTabbedActivity(final String user_email){
        DatabaseReference mDatabase = FirebaseDatabase.getInstance().getReference("users");

        mDatabase.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                String username = "";

                // Search username from email

                for (DataSnapshot dataValues : dataSnapshot.getChildren()){
                    if(user_email.equals(dataValues.child("email").getValue(String.class))){
                        username = dataValues.getKey();
                        break;
                    }
                }

                // check if username is not empty
                if(!username.equals("")){
                    // Save username in the app
                    MyApplication app = (MyApplication)getApplication();
                    app.setUser_profile(username);

                    Intent intent = new Intent(LoginActivity.this, TabbedActivity.class);
                    startActivity(intent);
                    finish();
                }else{
                    Log.d(TAG, "Failed to read email from database");
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });
    }
}

