package uoc.javier.allpay;

import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.support.annotation.NonNull;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.TextUtils;
import android.text.style.RelativeSizeSpan;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import uoc.javier.allpay.model.User;

public class NewUserActivity extends AppCompatActivity implements View.OnClickListener {
    private static final String TAG = "TFM:NewUserActivity";

    // UI references.
    private TextView tvLegend;
    private EditText etEmail;
    private EditText etName;
    private EditText etSurname;
    private EditText etUsername;
    private EditText etPassword;
    private EditText etRepeatPassword;
    private Button buttonSave;
    private Button buttonReturn;
    private Button buttonRemove;

    // Receive the item from UserMovementsFragment
    private Bundle bundle;
    private String username;

    // Firebase
    DatabaseReference mDatabase;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_user);

        // Set views
        tvLegend = (TextView) findViewById(R.id.tv_legend);

        etEmail = (EditText) findViewById(R.id.et_email);
        etName = (EditText) findViewById(R.id.et_name);
        etSurname = (EditText) findViewById(R.id.et_surname);
        etUsername = (EditText) findViewById(R.id.et_username);
        etPassword = (EditText) findViewById(R.id.et_password);
        etRepeatPassword = (EditText) findViewById(R.id.et_repeat_password);

        buttonSave = (Button) findViewById(R.id.btn_save);
        buttonReturn = (Button) findViewById(R.id.btn_return);
        buttonRemove = (Button) findViewById(R.id.btn_remove);

        mDatabase = FirebaseDatabase.getInstance().getReference("users");

        // Set listeners
        buttonSave.setOnClickListener(this);
        buttonReturn.setOnClickListener(this);
        buttonRemove.setOnClickListener(this);

        // Edit user
        bundle = getIntent().getExtras();
        if(bundle != null) {
            tvLegend.setText(getResources().getString(R.string.edit_profile));
            tvLegend.setTextColor(Color.BLACK);

            buttonSave.setText(getResources().getString(R.string.edit));
            buttonRemove.setEnabled(true);
            buttonRemove.setVisibility(View.VISIBLE);

            username = bundle.getString("username_profile");
            downloadThisUser();
        }
    }

    public void downloadThisUser(){
        // Download user from Firebase
        mDatabase.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                User user = dataSnapshot.child(username).getValue(User.class);

                // Exception after removing user
                try {
                    etEmail.setText(user.getEmail());
                    etName.setText(user.getName());
                    etSurname.setText(user.getSurname());
                    etUsername.setText(dataSnapshot.child(username).getKey());

                    // it is the key in Firebase so it can't be edited
                    etUsername.setFocusable(false);
                }catch (Exception e){

                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });
    }

    public void onClick(View v){
        if(v == buttonSave){
            // Check that each requiered field is filled
            if(!checkRequeriedFields(new EditText[]{etEmail, etName, etSurname, etUsername, etPassword, etRepeatPassword})){
                showToast(getResources().getString(R.string.complete_fields));
            }
            else{
                View focusView = null;
                focusView = attempCreateUser();
                if (focusView != null) {
                    // There was an error; don't attempt login and focus the first
                    // form field with an error.
                    focusView.requestFocus();
                } else {
                    // sign up new user or edit user
                    User user = new User(etEmail.getText().toString(),
                            etPassword.getText().toString(),
                            etName.getText().toString(),
                            etSurname.getText().toString(),
                            etUsername.getText().toString());

                    mDatabase.child(etUsername.getText().toString()).setValue(user)
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                // sign up new user
                                if(buttonSave.getText().equals(getResources().getString(R.string.sign_up))){
                                    showToast(getResources().getString(R.string.add_user_right));
                                }
                                // edit user
                                else{
                                    showToast(getResources().getString(R.string.add_user_wrong));
                                }
                                goTabbedActivity();
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                // sign up new user
                                if (buttonSave.getText().equals(getResources().getString(R.string.sign_up))) {
                                    showToast(getResources().getString(R.string.add_user_wrong));
                                }
                                // edit user
                                else {
                                    showToast(getResources().getString(R.string.update_user_wrong));
                                }
                            }
                        });
                }
            }
        }

        else if(v == buttonReturn){
            // View profile
            if(bundle != null) {
                goTabbedActivity();
            }
            else {
                // Sign up, go to LoginActivity
                goLoginActivity();
            }
        }

        else if(v == buttonRemove) {
            // Ask to be sure and then remove the user
            askRemoveConfirmation();
        }
    }

    public void askRemoveConfirmation(){
        DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which){
                    case DialogInterface.BUTTON_POSITIVE:
                        removeUser();
                        break;

                    case DialogInterface.BUTTON_NEGATIVE:
                        break;
                }
            }
        };

        AlertDialog builder = new AlertDialog.Builder(this)
                .setMessage(getResources().getString(R.string.remove_user_confirmation))
                .setPositiveButton(getResources().getString(R.string.option_yes), dialogClickListener)
                .setNegativeButton(getResources().getString(R.string.option_no), dialogClickListener).show();

        TextView textView = (TextView) builder.findViewById(android.R.id.message);
        textView.setTextSize(20);
    }

    public void removeUser() {
        mDatabase.child(username).removeValue()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        showToast(getResources().getString(R.string.remove_user_right));
                        goLoginActivity();
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        showToast(getResources().getString(R.string.remove_user_wrong));
                    }
                });
    }

    public void goTabbedActivity(){
        // Go to the main fragment of the TabbetActivity and clean all the fields
        Intent intent = new Intent(this, TabbedActivity.class);
        startActivity(intent);
    }

    public void goLoginActivity(){
        Intent intent = new Intent(this, LoginActivity.class);
        startActivity(intent);
        finish();
    }

    public boolean checkRequeriedFields(EditText[] fields){
        EditText currentField;
        for(int i=0; i<fields.length; i++){
            currentField = fields[i];
            if(currentField.getText().toString().length() <=0){
                currentField.setError(getResources().getString(R.string.complete_fields));
                return false;
            }
        }
        return true;
    }

    private View attempCreateUser(){

        // Store values at the time of the login attempt.
        String email = etEmail.getText().toString();
        String password = etPassword.getText().toString();
        String username = etUsername.getText().toString();

        View focusView = null;

        // Check for a valid password, if the user entered one.
        if (!TextUtils.isEmpty(password) && !isPasswordValid(password)) {
            etPassword.setError(getString(R.string.error_invalid_password));
            focusView = etPassword;
        }

        // Check both password are the same
        if(!password.equals(etRepeatPassword.getText().toString())){
            etRepeatPassword.setError(getString(R.string.same_passwords));
            focusView = etRepeatPassword;
        }

        // Check for a valid email address.
        if (TextUtils.isEmpty(email)) {
            etEmail.setError(getString(R.string.error_field_required));
            focusView = etEmail;
        } else if (!isEmailValid(email)) {
            etEmail.setError(getString(R.string.error_invalid_email));
            focusView = etEmail;
        }

        // Check for a valid username
        if (TextUtils.isEmpty(username)) {
            etUsername.setError(getString(R.string.error_field_required));
            focusView = etUsername;
        } else if (!isUsernameValid(username)) {
            etUsername.setError(getString(R.string.error_invalid_username));
            focusView = etUsername;
        }
        return focusView;
    }

    private boolean isUsernameValid(String username){
        return !(username.contains(".") || username.contains("$") || username.contains("#") || username.contains("[") || username.contains("]") || username.contains("/"));
    }

    private boolean isEmailValid(String email) {
        return email.contains("@");
    }

    private boolean isPasswordValid(String password) {
        return password.length() > 4;
    }

    public void showToast(String str){
        // Configuration size of text
        SpannableString ss =  new SpannableString(str);
        ss.setSpan(new RelativeSizeSpan(1.5f), 0, ss.length(), 0);

        // Show the message
        Toast.makeText(this, ss, Toast.LENGTH_LONG).show();
    }

}
