package uoc.javier.allpay;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AlertDialog;
import android.support.v7.util.DiffUtil;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.text.SpannableString;
import android.text.style.RelativeSizeSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import uoc.javier.allpay.model.Goal;
import uoc.javier.allpay.model.Item;
import uoc.javier.allpay.model.Payment;
import uoc.javier.allpay.utils.GoalDiffCallback;
import uoc.javier.allpay.utils.ItemDiffCallback;
import uoc.javier.allpay.utils.PaymentDiffCallback;


/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link PaymentsFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 * Use the {@link PaymentsFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class PaymentsFragment extends Fragment {
    private static final String TAG = "TFM:Payments";

    // TODO: Rename parameter arguments, choose names that match
    // the fragment initialization parameters, e.g. ARG_ITEM_NUMBER
    private static final String ARG_PARAM1 = "param1";
    private static final String ARG_PARAM2 = "param2";

    // TODO: Rename and change types of parameters
    private String mParam1;
    private String mParam2;

    private OnFragmentInteractionListener mListener;

    private RecyclerView recyclerView;
    private PaymentsFragment.SimpleItemRecyclerViewAdapter adapter;
    private SwipeRefreshLayout swipeContainer;

    // Firebase
    private FirebaseDatabase database;
    DatabaseReference myRef;

    // Total balance between the money a user has spent (being a payer) and the money he has received (being a receiver)
    Map<String, Double> user_balance = new HashMap<>();

    public PaymentsFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @param param1 Parameter 1.
     * @param param2 Parameter 2.
     * @return A new instance of fragment PaymentsFragment.
     */
    // TODO: Rename and change types and number of parameters
    public static PaymentsFragment newInstance(String param1, String param2) {
        PaymentsFragment fragment = new PaymentsFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PARAM1, param1);
        args.putString(ARG_PARAM2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            mParam1 = getArguments().getString(ARG_PARAM1);
            mParam2 = getArguments().getString(ARG_PARAM2);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_payments, container, false);
    }

    @Override
    public void onViewCreated(final View view, Bundle savedInstanceState) {
        recyclerView = (RecyclerView)view.findViewById(R.id.payments_list);
        StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(1, StaggeredGridLayoutManager.VERTICAL);
        recyclerView.setLayoutManager(staggeredGridLayoutManager);
        setupRecyclerView(recyclerView);

        // Download items from Firebase
        database = FirebaseDatabase.getInstance();

        swipeContainer = (SwipeRefreshLayout) view.findViewById(R.id.swipeContainer);
        // Setup refresh listener which triggers new data loading
        swipeContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refreshItemsList();
            }
        });

        // Configure the refreshing colors
        swipeContainer.setColorSchemeResources(android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);

        setupUserBalance();

        downloadItems();

        // Functionality --> All payments were completed, so clean the item list
        FloatingActionButton fab = (FloatingActionButton) view.findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                askRemoveAllItemsConfirmation();
            }
        });
    }

    public void askRemoveAllItemsConfirmation(){
        DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which){
                    case DialogInterface.BUTTON_POSITIVE:
                        removeAllItems();
                        break;

                    case DialogInterface.BUTTON_NEGATIVE:
                        break;
                }
            }
        };

        AlertDialog builder = new AlertDialog.Builder(getActivity())
                .setMessage(getResources().getString(R.string.remove_all_items_confirmation))
                .setTitle(getResources().getString(R.string.alert_capital))
                .setPositiveButton(getResources().getString(R.string.option_yes), dialogClickListener)
                .setNegativeButton(getResources().getString(R.string.option_no), dialogClickListener).show();

        TextView textView = (TextView) builder.findViewById(android.R.id.message);
        textView.setTextSize(20);
    }

    public void removeAllItems() {
        myRef.removeValue()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void aVoid) {
                        showToast(getResources().getString(R.string.remove_all_items_right));
                        goTabbedActivity();
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        showToast(getResources().getString(R.string.remove_all_items_wrong));
                    }
                });
    }

    public void goTabbedActivity(){
        // Go to the main fragment of the TabbetActivity and clean all the fields
        Intent intent = new Intent(getActivity(), TabbedActivity.class);
        startActivity(intent);
    }

    private void downloadItems() {
        swipeContainer.setRefreshing(true);

        myRef = database.getReference("items");
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getItemsFromDataSnapshot(dataSnapshot);
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });

        DatabaseReference connectedRef = database.getReference(".info/connected");
        connectedRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot snapshot) {
                boolean connected = snapshot.getValue(Boolean.class);
                if (!connected) {
                    Log.d(TAG, "Failed to connect");
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG, "Listener was cancelled");
            }
        });
    }

    private void refreshItemsList() {
        if (myRef != null) {
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    getItemsFromDataSnapshot(dataSnapshot);
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    // Failed to read value
                    Log.d(TAG, "Failed to read value.", databaseError.toException());
                }
            });
        } else {
            Log.d(TAG, "My Ref null");
        }
    }

    private void getItemsFromDataSnapshot(DataSnapshot dataSnapshot) {
        // This method is called once with the initial value and again
        // whenever data at this location is updated.
        ArrayList<Payment> values = new ArrayList<>();

        List<String> payers_users;
        List<String> receivers_users;

        Double each_price_paid;
        Double each_price_receive;

        // Reset payer_payments values
        for (Map.Entry<String, Double> entry : user_balance.entrySet()) {
            entry.setValue((double)0);
        }

        // Get each item
        for (DataSnapshot dataValues : dataSnapshot.getChildren()){
            payers_users = new ArrayList<>();
            receivers_users = new ArrayList<>();

            // Who has PAID the item
            for (DataSnapshot payer : dataValues.child("payers_users").getChildren()) {
                payers_users.add(payer.getKey());
            }

            each_price_paid = Double.valueOf(dataValues.child("price").getValue(String.class)) / payers_users.size();

            // How much each has paid
            for(int i=0; i<payers_users.size(); i++){
                user_balance.put(payers_users.get(i), user_balance.get(payers_users.get(i)) + each_price_paid);
            }

            // Who has RECEIVED the item
            for (DataSnapshot receiver : dataValues.child("receivers_users").getChildren()) {
                receivers_users.add(receiver.getKey());
            }

            each_price_receive = Double.valueOf(dataValues.child("price").getValue(String.class)) / receivers_users.size();

            // How much each has received
            for(int i=0; i<receivers_users.size(); i++){
                user_balance.put(receivers_users.get(i), user_balance.get(receivers_users.get(i)) - each_price_receive);
            }

        }

        //Log.d(TAG, "TOTAL "+user_balance.toString());

        // Create all the payments
        Payment payment;
        for(Map.Entry<String, Double> entry : user_balance.entrySet()){
            payment = new Payment();

            payment.setUsername(entry.getKey());
            payment.setBalance(entry.getValue());

            values.add(payment);
        }

        // Update values
        adapter.setItems(values);
        swipeContainer.setRefreshing(false);

        // Check if the fragment is currently added to its activity.
        if(isAdded())
            Snackbar.make(swipeContainer, getResources().getString(R.string.update_payments), Snackbar.LENGTH_LONG).show();
    }

    public void setupUserBalance(){
        // Download users from Firebase
        DatabaseReference databaseUsers;
        databaseUsers = FirebaseDatabase.getInstance().getReference("users");

        databaseUsers.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                for(DataSnapshot user : dataSnapshot.getChildren()){
                    user_balance.put(user.getKey(), (double) 0);
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        adapter = new PaymentsFragment.SimpleItemRecyclerViewAdapter(new ArrayList<Payment>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<PaymentsFragment.SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<Payment> mValues;

        public SimpleItemRecyclerViewAdapter(List<Payment> payments) {
            mValues = payments;
        }

        public void setItems(List<Payment> payments) {
            DiffUtil.DiffResult diffResult = DiffUtil.calculateDiff(new PaymentDiffCallback(mValues, payments));
            diffResult.dispatchUpdatesTo(this);
            mValues = payments;
        }

        @Override
        public PaymentsFragment.SimpleItemRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext()).inflate(R.layout.payments_list_content, parent, false);
            return new PaymentsFragment.SimpleItemRecyclerViewAdapter.ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final PaymentsFragment.SimpleItemRecyclerViewAdapter.ViewHolder holder, final int position) {
            holder.mPayment = mValues.get(position);
            holder.mUser.setText(holder.mPayment.getUsername());

            if(holder.mPayment.getBalance() < 0.0) {
                holder.mBalance.setTextColor(Color.RED);
                holder.mBalance.setText(String.valueOf(String.format("%.2f", holder.mPayment.getBalance())) + " €");
            }
            else if (holder.mPayment.getBalance() > 0.0){
                holder.mBalance.setTextColor(Color.parseColor("#009933"));
                holder.mBalance.setText("+" + String.valueOf(String.format("%.2f", holder.mPayment.getBalance())) + " €");

            }
            else{
                holder.mBalance.setText(String.valueOf(String.format("%.2f", holder.mPayment.getBalance())) + " €");
            }
        }

        @Override
        public int getItemCount() {
            return mValues.size();
        }

        class ViewHolder extends RecyclerView.ViewHolder {
            final View mView;
            final TextView mUser;
            final TextView mBalance;
            Payment mPayment;

            ViewHolder(View view) {
                super(view);
                mView = view;
                mUser = (TextView) view.findViewById(R.id.user_content);
                mBalance = (TextView) view.findViewById(R.id.balance_content);
            }
        }
    }

    // TODO: Rename method, update argument and hook method into UI event
    public void onButtonPressed(Uri uri) {
        if (mListener != null) {
            mListener.onFragmentInteraction(uri);
        }
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            //throw new RuntimeException(context.toString() + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    public void showToast(String str){
        // Configuration size of text
        SpannableString ss =  new SpannableString(str);
        ss.setSpan(new RelativeSizeSpan(1.5f), 0, ss.length(), 0);

        // Show the message
        Toast.makeText(getActivity(), ss, Toast.LENGTH_LONG).show();
    }
}
