package uoc.javier.allpay;

import android.content.Intent;
import android.net.Uri;
import android.support.design.widget.TabLayout;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;

import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.style.RelativeSizeSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.mikepenz.materialdrawer.AccountHeader;
import com.mikepenz.materialdrawer.AccountHeaderBuilder;
import com.mikepenz.materialdrawer.Drawer;
import com.mikepenz.materialdrawer.DrawerBuilder;
import com.mikepenz.materialdrawer.model.DividerDrawerItem;
import com.mikepenz.materialdrawer.model.PrimaryDrawerItem;
import com.mikepenz.materialdrawer.model.ProfileDrawerItem;
import com.mikepenz.materialdrawer.model.interfaces.IDrawerItem;
import com.mikepenz.materialdrawer.model.interfaces.IProfile;

import uoc.javier.allpay.model.User;

public class TabbedActivity extends AppCompatActivity {
    private static final String TAG = "TFM:TabbedActivity";

    /**
     * The {@link android.support.v4.view.PagerAdapter} that will provide
     * fragments for each of the sections. We use a
     * {@link FragmentPagerAdapter} derivative, which will keep every
     * loaded fragment in memory. If this becomes too memory intensive, it
     * may be best to switch to a
     * {@link android.support.v4.app.FragmentStatePagerAdapter}.
     */
    private SectionsPagerAdapter mSectionsPagerAdapter;

    /**
     * The {@link ViewPager} that will host the section contents.
     */
    private ViewPager mViewPager;

    // items Material Drawer
    private final static int ITEM_HISTORY = 1;
    private final static int GROUP_OPTIONS = 3;
    private final static int EDIT_PROFILE = 5;
    private final static int PAY = 7;
    private final static int LOG_OUT = 9;

    private User user_profile;
    private String username_profile;
    private Toolbar toolbar;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_tabbed);

        toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Get username
        MyApplication app = (MyApplication)getApplication();
        username_profile = app.getUser_profile();

        // Testing with diferent user
        //username_profile = "prueba";

        // Get user data from Firebase
        DatabaseReference mDatabase = FirebaseDatabase.getInstance().getReference("users");

        mDatabase.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                // Search username from email
                for (DataSnapshot dataValues : dataSnapshot.getChildren()){
                    if(username_profile.equals(dataValues.getKey())){
                        user_profile = dataValues.getValue(User.class);
                        // Set up Material drawer
                        setupMaterialDrawer(toolbar);
                        break;
                    }
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });

        // Create the adapter that will return a fragment for each of the three
        // primary sections of the activity.
        mSectionsPagerAdapter = new SectionsPagerAdapter(getSupportFragmentManager());

        // Set up the ViewPager with the sections adapter.
        mViewPager = (ViewPager) findViewById(R.id.container);
        mViewPager.setAdapter(mSectionsPagerAdapter);

        TabLayout tabLayout = (TabLayout) findViewById(R.id.tabs);

        mViewPager.addOnPageChangeListener(new TabLayout.TabLayoutOnPageChangeListener(tabLayout));
        tabLayout.addOnTabSelectedListener(new TabLayout.ViewPagerOnTabSelectedListener(mViewPager));

        tabLayout.setupWithViewPager(mViewPager);
    }

    private void setupMaterialDrawer(Toolbar toolbar) {
        //Items
        PrimaryDrawerItem item_eprofile = new PrimaryDrawerItem().withIdentifier(EDIT_PROFILE).withName(R.string.edit_profile);
        PrimaryDrawerItem item_group = new PrimaryDrawerItem().withIdentifier(GROUP_OPTIONS).withName(R.string.group_options);
        PrimaryDrawerItem item_pay = new PrimaryDrawerItem().withIdentifier(PAY).withName(R.string.pay);
        PrimaryDrawerItem item_lout = new PrimaryDrawerItem().withIdentifier(LOG_OUT).withName(R.string.log_out);
        PrimaryDrawerItem item_history = new PrimaryDrawerItem().withIdentifier(ITEM_HISTORY).withName(R.string.items_history_capital);

        String account_name = user_profile.getName() + " " + user_profile.getSurname();

        // Create the AccountHeader
        AccountHeader accountHeader = new AccountHeaderBuilder()
                .withActivity(this)
                .withHeaderBackground(R.color.md_blue_400)
                .addProfiles(
                        new ProfileDrawerItem().withName(account_name).withEmail(user_profile.getEmail())
                )
                .withOnAccountHeaderListener(new AccountHeader.OnAccountHeaderListener() {
                    @Override
                    public boolean onProfileChanged(View view, IProfile profile, boolean currentProfile) {
                        return false;
                    }
                })
                .build();

        // Create the drawer and remember the `Drawer` result object
        Drawer result = new DrawerBuilder()
                .withActivity(this)
                .withToolbar(toolbar)
                .withAccountHeader(accountHeader)
                .addDrawerItems(
                        item_history,
                        new DividerDrawerItem(),
                        item_group,
                        new DividerDrawerItem(),
                        item_eprofile,
                        new DividerDrawerItem(),
                        item_pay,
                        new DividerDrawerItem(),
                        item_lout
                )
                .withOnDrawerItemClickListener(new Drawer.OnDrawerItemClickListener() {
                    @Override
                    public boolean onItemClick(View view, int position, IDrawerItem drawerItem) {
                        Intent intent;
                        switch (position) {
                            case EDIT_PROFILE:
                                // Edit profile
                                intent = new Intent(TabbedActivity.this, NewUserActivity.class);
                                // Send the current user to differ between create new user and edit an user
                                intent.putExtra("username_profile", username_profile);
                                startActivity(intent);
                                finish();
                                break;
                            case GROUP_OPTIONS:
                                intent = new Intent(TabbedActivity.this, GroupActivity.class);
                                startActivity(intent);
                                finish();
                                break;
                            case PAY:
                                // Pay through paypal
                                Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse("https://www.paypal.com"));
                                startActivity(browserIntent);
                                break;
                            case LOG_OUT:
                                // Go to LoginActivity
                                intent = new Intent(TabbedActivity.this, LoginActivity.class);
                                startActivity(intent);
                                finish();
                                break;
                            case ITEM_HISTORY:
                                // Go to ItemsHistoricalActivity
                                intent = new Intent(TabbedActivity.this, ItemsHistoricalActivity.class);
                                startActivity(intent);
                                finish();
                                break;
                            default:
                                break;
                        }
                        return true;
                    }
                })
                .withSelectedItem(-1)
                .build();
    }

    public void showToast(String str){
        // Configuration size of text
        SpannableString ss =  new SpannableString(str);
        ss.setSpan(new RelativeSizeSpan(1.5f), 0, ss.length(), 0);

        // Show the message
        Toast.makeText(TabbedActivity.this, ss, Toast.LENGTH_LONG).show();
    }

    /**
     * A placeholder fragment containing a simple view.
     */
    public static class PlaceholderFragment extends Fragment {
        /**
         * The fragment argument representing the section number for this
         * fragment.
         */
        private static final String ARG_SECTION_NUMBER = "section_number";

        public PlaceholderFragment() {
        }

        /**
         * Returns a new instance of this fragment for the given section
         * number.
         */
        public static PlaceholderFragment newInstance(int sectionNumber) {
            PlaceholderFragment fragment = new PlaceholderFragment();
            Bundle args = new Bundle();
            args.putInt(ARG_SECTION_NUMBER, sectionNumber);
            fragment.setArguments(args);
            return fragment;
        }

        @Override
        public View onCreateView(LayoutInflater inflater, ViewGroup container,
                                 Bundle savedInstanceState) {
            View rootView = inflater.inflate(R.layout.fragment_tabbed, container, false);
            TextView textView = (TextView) rootView.findViewById(R.id.section_label);
            textView.setText(getString(R.string.section_format, getArguments().getInt(ARG_SECTION_NUMBER)));
            return rootView;
        }
    }

    /**
     * A {@link FragmentPagerAdapter} that returns a fragment corresponding to
     * one of the sections/tabs/pages.
     */
    public class SectionsPagerAdapter extends FragmentPagerAdapter {

        public SectionsPagerAdapter(FragmentManager fm) {
            super(fm);
        }

        @Override
        public Fragment getItem(int position) {
            // getItem is called to instantiate the fragment for the given page.
            // Return a PlaceholderFragment (defined as a static inner class below).
//            return PlaceholderFragment.newInstance(position + 1);

            Fragment fragment;
            switch(position) {
                case 0:
                    fragment = new UserMovementsFragment();
                    break;
                case 1:
                    fragment = new NewItemFragment();
                    break;
                case 2:
                    fragment = new PaymentsFragment();
                    break;
                case 3:
                    fragment = new GoalsFragment();
                    break;
                default:
                    fragment = PlaceholderFragment.newInstance(position + 1);
            }
            return fragment;
        }

        @Override
        public int getCount() {
            return 4;
        }

        @Override
        public CharSequence getPageTitle(int position) {
            // TODO: añadir iconos en lugar de nombres
            switch (position) {
                case 0:
                    return getResources().getString(R.string.last_movements);
                case 1:
                    return getResources().getString(R.string.new_item_tabbed);
                case 2:
                    return getResources().getString(R.string.payments);
                case 3:
                    return getResources().getString(R.string.goal_tabbed);
            }
            return null;
        }
    }
}
