package uoc.javier.allpay;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.util.DiffUtil;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.text.SpannableString;
import android.text.style.RelativeSizeSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import uoc.javier.allpay.model.Item;
import uoc.javier.allpay.utils.ItemDiffCallback;


/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link UserMovementsFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 * Use the {@link UserMovementsFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class UserMovementsFragment extends Fragment {
    private static final String TAG = "TFM:UserMovements";

    // TODO: Rename parameter arguments, choose names that match
    // the fragment initialization parameters, e.g. ARG_ITEM_NUMBER
    private static final String ARG_PARAM1 = "param1";
    private static final String ARG_PARAM2 = "param2";

    // TODO: Rename and change types of parameters
    private String mParam1;
    private String mParam2;

    private OnFragmentInteractionListener mListener;

    private RecyclerView recyclerView;
    private SimpleItemRecyclerViewAdapter adapter;

    private SwipeRefreshLayout swipeContainer;

    // Firebase
    private FirebaseDatabase database;
    private DatabaseReference myRef;

    // Who pay next
    // Map with <username, total number of payments made>
    Map<String, Integer> payer_payments = new HashMap<>();

    public UserMovementsFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @param param1 Parameter 1.
     * @param param2 Parameter 2.
     * @return A new instance of fragment UserMovementsFragment.
     */
    // TODO: Rename and change types and number of parameters
    public static UserMovementsFragment newInstance(String param1, String param2) {
        UserMovementsFragment fragment = new UserMovementsFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PARAM1, param1);
        args.putString(ARG_PARAM2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            mParam1 = getArguments().getString(ARG_PARAM1);
            mParam2 = getArguments().getString(ARG_PARAM2);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_user_movements, container, false);
    }

    @Override
    public void onViewCreated(final View view, Bundle savedInstanceState) {
        recyclerView = (RecyclerView)view.findViewById(R.id.usermovements_list);
        StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(1, StaggeredGridLayoutManager.VERTICAL);
        recyclerView.setLayoutManager(staggeredGridLayoutManager);
        setupRecyclerView(recyclerView);

        database = FirebaseDatabase.getInstance();

        swipeContainer = (SwipeRefreshLayout) view.findViewById(R.id.swipeContainer);
        // Setup refresh listener which triggers new data loading
        swipeContainer.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refreshItemsList();
            }
        });

        // Configure the refreshing colors
        swipeContainer.setColorSchemeResources(android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);

        // Necessary to know who has not paid yet
        setupPayerPayments();

        downloadItems();

        // Functionality --> Who pay the next one?
        FloatingActionButton fab = (FloatingActionButton) view.findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
               whoPayNext();
            }
        });
    }

    public void setupPayerPayments(){
        // Download users from Firebase
        DatabaseReference databaseUsers;
        databaseUsers = FirebaseDatabase.getInstance().getReference("users");

        databaseUsers.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                for(DataSnapshot user : dataSnapshot.getChildren()){
                    payer_payments.put(user.getKey(), 0);
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });
    }

    // Search for the user who has not paid for the longest time
    private void whoPayNext(){
        // Each time a user pays is updated inside getItemsFromDataSnapshot
        //Log.d(TAG, "WHO PAY NEXT "+payer_payments.toString());

        Stream<Map.Entry<String,Integer>> sorted =
                payer_payments.entrySet().stream()
                        .sorted(Map.Entry.comparingByValue());

        createSpannableString(sorted.findFirst().get().getKey());
    }

    public void createSpannableString(String message){
        // Texts size
        float ss_proportion = 1.5f;
        String title = getResources().getString(R.string.who_pay_next);

        SpannableString ssMessage = new SpannableString("\n"+message);
        ssMessage.setSpan(new RelativeSizeSpan(ss_proportion), 0, ssMessage.length(), 0);

        SpannableString ssTitle = new SpannableString(title);
        ssTitle.setSpan(new RelativeSizeSpan(ss_proportion), 0, ssTitle.length(), 0);

        // Build dialog
        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder
                .setTitle(ssTitle)
                .setMessage(ssMessage)
                .setCancelable(false)
                .setIconAttribute(android.R.attr.alertDialogIcon)
                .setPositiveButton(getResources().getString(R.string.ok), new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                    }
                });

        AlertDialog alert = builder.create();
        alert.show();
    }

    private void downloadItems() {
        swipeContainer.setRefreshing(true);

        myRef = database.getReference("items");
        // Read from the database
        myRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getItemsFromDataSnapshot(dataSnapshot);
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.d(TAG, "Failed to read value", error.toException());
            }
        });

        DatabaseReference connectedRef = database.getReference(".info/connected");
        connectedRef.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot snapshot) {
                boolean connected = snapshot.getValue(Boolean.class);
                if (!connected) {
                    Log.d(TAG, "Failed to connect");
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG, "Listener was cancelled");
            }
        });
    }

    private void refreshItemsList() {
        if (myRef != null) {
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    getItemsFromDataSnapshot(dataSnapshot);
                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    // Failed to read value
                    Log.d(TAG, "Failed to read value.", databaseError.toException());
                }
            });
        } else {
            Log.d(TAG, "My Ref null");
        }
    }

    private void getItemsFromDataSnapshot(DataSnapshot dataSnapshot) {
        // This method is called once with the initial value and again
        // whenever data at this location is updated.

        //Log.d("TAG",dataSnapshot.getValue().toString());

        ArrayList<Item> values = new ArrayList<>();

        Item item;
        List<String> payers_users;
        List<String> receivers_users;

        // Reset payer_payments values
        for (Map.Entry<String, Integer> entry : payer_payments.entrySet()) {
            entry.setValue(0);
        }

        // Get each item
        for (DataSnapshot dataValues : dataSnapshot.getChildren()){
            item = new Item();
            payers_users = new ArrayList<>();
            receivers_users = new ArrayList<>();

            item.setUid(dataValues.getKey());

            // Log.d(TAG, dataValues.toString());
            // Log.d(TAG, "key " + dataValues.child("price").getValue(String.class)); // GET CADA CAMPO INDIVIDUAL
            item.setDescription(dataValues.child("description").getValue(String.class));
            item.setPrice(dataValues.child("price").getValue(String.class));
            item.setType(dataValues.child("type").getValue(String.class));

            for(DataSnapshot payer : dataValues.child("payers_users").getChildren()){
//                Log.d(TAG, "hijo " + payers.getKey().toString());
                payers_users.add(payer.getKey());

                // Functionality "Whoy pay next"
                payer_payments.put(payer.getKey(), payer_payments.get(payer.getKey()) + 1);
            }

            for(DataSnapshot receiver : dataValues.child("receivers_users").getChildren()){
//                Log.d(TAG, "hijo " + receivers.getKey().toString());
                receivers_users.add(receiver.getKey());
            }

            item.setPayers_users(payers_users);
            item.setReceivers_users(receivers_users);

            values.add(item);
        }

        // To show the last ones first
        Collections.reverse(values);

        // Update values
        adapter.setItems(values);
        swipeContainer.setRefreshing(false);

        // Check if the fragment is currently added to its activity.
        if(isAdded())
            Snackbar.make(swipeContainer, getResources().getString(R.string.update_items), Snackbar.LENGTH_LONG).show();
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
//        adapter = new SimpleItemRecyclerViewAdapter(Item.ITEMS_EXAMPLE, getContext());
        adapter = new SimpleItemRecyclerViewAdapter(new ArrayList<Item>());
        recyclerView.setAdapter(adapter);
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<Item> mValues;


        public SimpleItemRecyclerViewAdapter(List<Item> items) {
            mValues = items;
        }

        public void setItems(List<Item> items) {
            DiffUtil.DiffResult diffResult = DiffUtil.calculateDiff(new ItemDiffCallback(mValues, items));
            diffResult.dispatchUpdatesTo(this);
            mValues = items;
        }

        @Override
        public UserMovementsFragment.SimpleItemRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = null;

            view = LayoutInflater.from(parent.getContext()).inflate(R.layout.usermovements_list_content, parent, false);
            return new UserMovementsFragment.SimpleItemRecyclerViewAdapter.ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mItem = mValues.get(position);
            holder.mSubject.setText(holder.mItem.getDescription());
            holder.mPrice.setText(holder.mItem.getPrice() + " €");

            // Delete "[" and "]" characters
            holder.mFrom.setText(holder.mItem.getPayers_users().toString().replace("[", "").replace("]", ""));
            holder.mTo.setText(holder.mItem.getReceivers_users().toString().replace("[", "").replace("]", ""));

            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Context context = v.getContext();
                    Intent intent = new Intent(context, EditItemActivity.class);
                    intent.putExtra("item_uid", holder.mItem.getUid());
                    context.startActivity(intent);
                }
            });
        }

        @Override
        public int getItemCount() {
            return mValues.size();
        }

        class ViewHolder extends RecyclerView.ViewHolder {
            final View mView;
            final TextView mFrom;
            final TextView mSubject;
            final TextView mPrice;
            final TextView mTo;
            Item mItem;

            ViewHolder(View view) {
                super(view);
                mView = view;
                mFrom = (TextView) view.findViewById(R.id.from_content);
                mSubject = (TextView) view.findViewById(R.id.subject_content);
                mPrice = (TextView) view.findViewById(R.id.price_content);
                mTo = (TextView) view.findViewById(R.id.to_content);
            }
        }

    }

    // Fragment default methods
    // TODO: Rename method, update argument and hook method into UI event
    public void onButtonPressed(Uri uri) {
        if (mListener != null) {
            mListener.onFragmentInteraction(uri);
        }
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            //throw new RuntimeException(context.toString() + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    public void showToast(String str){
        // Configuration size of text
        SpannableString ss =  new SpannableString(str);
        ss.setSpan(new RelativeSizeSpan(1.5f), 0, ss.length(), 0);

        // Show the message
        Toast.makeText(getActivity(), ss, Toast.LENGTH_LONG).show();
    }
}
